#include "CefBrowserEventHandler.h"
//#include "stdafx.h"
#include <sstream>
#include <string>
#include "include/cef_app.h"
#include "include/wrapper/cef_closure_task.h"
#include "include/wrapper/cef_helpers.h"
#include "CefDialog.h"


CCefBrowserEventHandler::CCefBrowserEventHandler(CefDialog* pCefFrame)
	:m_bIsClosing(false)
	, m_pCefWnd(pCefFrame)
{

}

CCefBrowserEventHandler::~CCefBrowserEventHandler()
{

}

CefRefPtr<CefDisplayHandler> CCefBrowserEventHandler::GetDisplayHandler()
{
	return this;
}

CefRefPtr<CefLifeSpanHandler> CCefBrowserEventHandler::GetLifeSpanHandler()
{
	return this;
}

CefRefPtr<CefLoadHandler> CCefBrowserEventHandler::GetLoadHandler() 
{
	return this;
}

void CCefBrowserEventHandler::OnTitleChange(CefRefPtr<CefBrowser> browser, const CefString& title) 
{
	
}

void CCefBrowserEventHandler::OnLoadError(CefRefPtr<CefBrowser> browser, CefRefPtr<CefFrame> frame, ErrorCode errorCode, 
										  const CefString& errorText, const CefString& failedUrl) 
{
	CEF_REQUIRE_UI_THREAD();
	if (ERR_ABORTED == errorCode)
		return ;

	std::stringstream ss;
	ss <<	"<html><body bgcolor=\"white\">"
			"<h2>Failed to load URL " << std::string(failedUrl) <<
			" with error " << std::string(errorText) << " (" << errorCode <<
			").</h2></body></html>";
	frame->LoadString(ss.str(), failedUrl);
}

void CCefBrowserEventHandler::OnLoadEnd(CefRefPtr<CefBrowser> browser, CefRefPtr<CefFrame> frame, int httpStatusCode)
{
	std::wstring url = browser->GetMainFrame()->GetURL();

	m_pCefWnd->CEFInvalidateRect();
	return __super::OnLoadEnd(browser, frame, httpStatusCode);
}

void CCefBrowserEventHandler::OnLoadStart(CefRefPtr<CefBrowser> browser, CefRefPtr<CefFrame> frame, TransitionType transition_typ)
{
	std::wstring url = browser->GetMainFrame()->GetURL();

	return __super::OnLoadStart(browser, frame, transition_typ);
}

void CCefBrowserEventHandler::OnAfterCreated(CefRefPtr<CefBrowser> browser) 
{
	CEF_REQUIRE_UI_THREAD();

	//base::AutoLock lock_scope(lock_);

	AutoLock lock_scope(this);
	if (!m_Browser.get())
	{
		m_Browser = browser;
		m_BrowserId = browser->GetIdentifier();
	}
	 

}

bool CCefBrowserEventHandler::DoClose(CefRefPtr<CefBrowser> browser)
{
	CEF_REQUIRE_UI_THREAD();

	//base::AutoLock lock_scope(lock_);
	AutoLock lock_scope(this);


	if(m_BrowserId == browser->GetIdentifier()) 
	{
		// Set a flag to indicate that the window close should be allowed.
		m_bIsClosing = true;
	}

	return false;
}

void CCefBrowserEventHandler::OnBeforeClose(CefRefPtr<CefBrowser> browser) 
{
	CEF_REQUIRE_UI_THREAD();

	//base::AutoLock lock_scope(lock_);
	AutoLock lock_scope(this);

	if(m_BrowserId == browser->GetIdentifier())  
	{
		m_Browser = NULL;
	}
}

bool CCefBrowserEventHandler::OnDragEnter(CefRefPtr<CefBrowser> browser,
										  CefRefPtr<CefDragData> dragData,
										  CefDragHandler::DragOperationsMask mask) 
{
	CEF_REQUIRE_UI_THREAD();

	// Forbid dragging of link URLs.
	if (mask & DRAG_OPERATION_LINK)
		return true;

	return false;
}

bool CCefBrowserEventHandler::OnProcessMessageReceived(CefRefPtr<CefBrowser> browser,
									  CefProcessId source_process,
									  CefRefPtr<CefProcessMessage> message) 
{
	const std::string& messageName = message->GetName();

	if(messageName == "login_msg")
	{
		// extract message
		CefRefPtr<CefListValue> args = message->GetArgumentList();
		CefString strUser = args->GetString(0);
		CefString strPassword = args->GetString(1);

		m_pCefWnd->CEFJsCallCpp(strUser, strPassword);
	}
	return false;
}

bool CCefBrowserEventHandler::OnBeforePopup(CefRefPtr<CefBrowser> browser,
	CefRefPtr<CefFrame> frame,
	const CefString& target_url,
	const CefString& target_frame_name,
	WindowOpenDisposition target_disposition,
	bool user_gesture,
	const CefPopupFeatures& popupFeatures,
	CefWindowInfo& windowInfo,
	CefRefPtr<CefClient>& client,
	CefBrowserSettings& settings,
	bool* no_javascript_access)
{
	switch (target_disposition)
	{
	case WOD_NEW_FOREGROUND_TAB:
	case WOD_NEW_BACKGROUND_TAB:
	case WOD_NEW_POPUP:
	case WOD_NEW_WINDOW:
		browser->GetMainFrame()->LoadURL(target_url);
		return true; //cancel create
	}
	return false;
}

void CCefBrowserEventHandler::OnBeforeDownload(
	CefRefPtr<CefBrowser> browser,
	CefRefPtr<CefDownloadItem> download_item,
	const CefString& suggested_name,
	CefRefPtr<CefBeforeDownloadCallback> callback)
{

	WCHAR sMyDocuPath[800] = { 0 };
	::SHGetSpecialFolderPathW(0, sMyDocuPath, CSIDL_PERSONAL, 0);

	wstring sDownloadPath = sMyDocuPath;
	sDownloadPath += L"\\";
	sDownloadPath += suggested_name.ToWString();

	wstring wstrExt = suggested_name.ToWString();
	string::size_type position = wstrExt.find(L".");


	if (position == -1)//ûҵ˵ļûչ,ǲĬļչȻغõļ޷鿴
	{
		sDownloadPath += L".jpg";
	}

	CefString sPath;
	sPath.FromWString(sDownloadPath);
	//Param [download_path] Set |download_path| to the full file path  for the download including the file name
	//Param [show_dialog] Set | show_dialog | to true if you do wish to show the default "Save As" dialog.

	callback->Continue(sDownloadPath, true);//һļȫ·ļ
}

void CCefBrowserEventHandler::OnDownloadUpdated(
	CefRefPtr<CefBrowser> browser,
	CefRefPtr<CefDownloadItem> download_item,
	CefRefPtr<CefDownloadItemCallback> callback)
{
	if (download_item->IsComplete())//
	{
		wstring sFilePath = download_item->GetFullPath().ToWString();//õļȫ·
	}
	
} 

bool CCefBrowserEventHandler::OnPreKeyEvent(CefRefPtr<CefBrowser> browser,
	const CefKeyEvent& event,
	CefEventHandle os_event,
	bool* is_keyboard_shortcut)
{
	if (event.type == KEYEVENT_RAWKEYDOWN)
	{
		if (event.windows_key_code == VK_F5)//F5ˢ
		{
			browser->Reload();
		}
	}
	

	return false;
}

bool CCefBrowserEventHandler::OnKeyEvent(CefRefPtr<CefBrowser> browser,
	const CefKeyEvent& event,
	CefEventHandle os_event)
{
	return false;
}

enum MyEnum
{
	MENU_ID_USER_OPENLINK = MENU_ID_USER_FIRST + 200,
	MENU_ID_USER_SHOWDEVTOOLS,
};

void CCefBrowserEventHandler::OnBeforeContextMenu(CefRefPtr<CefBrowser> browser, CefRefPtr<CefFrame> frame, CefRefPtr<CefContextMenuParams> params, CefRefPtr<CefMenuModel> model)
{
	model->Remove(MENU_ID_PRINT);
	model->Remove(MENU_ID_VIEW_SOURCE);

	if ((params->GetTypeFlags() & (CM_TYPEFLAG_PAGE | CM_TYPEFLAG_FRAME)) != 0) {
		// Add a separator if the menu already has items.
		if (model->GetCount() > 0)
		{
			model->RemoveAt(2);
			model->Remove(MENU_ID_BACK);//Ƴָ˵
			model->Remove(MENU_ID_FORWARD);
			//model->Clear();//ĬϵĲ˵
			//model->AddSeparator();//Ӳ˵ָ
#ifdef _DEBUG
			model->AddItem(MENU_ID_USER_SHOWDEVTOOLS, L"߹"); //"&Show DevTools");
#else

#endif
		}
	}
}

bool CCefBrowserEventHandler::OnContextMenuCommand(CefRefPtr<CefBrowser> browser, CefRefPtr<CefFrame> frame, CefRefPtr<CefContextMenuParams> params, int command_id, EventFlags event_flags)
{
	switch (command_id)
	{
	case MENU_ID_USER_SHOWDEVTOOLS:
	{
									  ShowDevelopTools(browser);
									  return true;
	}
	default:
		break;
	}

	return false;
}

void CCefBrowserEventHandler::ShowDevelopTools(CefRefPtr<CefBrowser> browser)
{
	CefWindowInfo windowInfo; 
	CefBrowserSettings settings;

#if defined(OS_WIN)
	//windowInfo.SetAsPopup(browser->GetHost()->GetWindowHandle(), "DevTools");
	windowInfo.SetAsPopup(NULL, "DevTools");

	//RECT rc = { 0, 0, 800, 600 };
	//windowInfo.SetAsChild(*theApp.m_pMainWnd, rc);
#endif

	browser->GetHost()->ShowDevTools(windowInfo, this, settings, CefPoint());
}